#!/usr/bin/python2

import urwid
import api_kareha
from data import Thread, Post

class KarehaReader(object):
    def __init__(self):
        gheader = urwid.AttrMap(urwid.Text("fucking-kareha-reader v0.1"), 'header')
        self.status = urwid.Text("Ready.")
        gstatusbar = urwid.AttrMap(self.status, 'footer')
        self.preview = urwid.Pile([urwid.Text('holi')])
    
        threadlist = api_kareha.get_threadlist()
        self.threadwalker = urwid.SimpleListWalker([ThreadItem(x) for x in threadlist])
         
        self.update_preview()
        listbox = urwid.ListBox(self.threadwalker)

        columns = urwid.Columns([ ('weight', 1, urwid.BoxAdapter(listbox, 50)  ),
                                  ('weight', 1, self.preview) ], 1)
        
        self.screen_front = urwid.Filler(columns, valign='top')
        self.mainframe = urwid.Frame(self.screen_front, header=gheader, footer=gstatusbar)
        palette = [('body', '', ''),
                   ('focus', 'yellow', 'dark blue'),
                   ('name', 'dark green', ''),
                   ('title', 'dark red', ''),
                   ('header', 'white', 'dark gray'),
                   ('footer', 'white', 'dark gray'),
                   ]
       
        self.mode = 'front'

        self.mainloop = urwid.MainLoop(self.mainframe, palette, unhandled_input=self.keypress)
        urwid.connect_signal(self.threadwalker, 'modified', self.update_preview)
        self.mainloop.run()

    def keypress(self, input):
        if self.mode is 'front':
            self.keypress_front(input)
        elif self.mode is 'thread':
            self.keypress_thread(input)
    
    def keypress_front(self, input):
        if input is 'enter':
            self.view_thread()
        elif input is 'q':
            raise urwid.ExitMainLoop()
    
    def keypress_thread(self, input):
        if input is 'q':
            self.view_front()
        elif input is 'r':
            self.show_replydialog()
    
    def set_status(self, txt):
        self.status.set_text(txt)
        self.mainloop.draw_screen()
    
    def update_preview(self):
        text = self.generate_thread_text(self.threadwalker.get_focus()[0].thread)
        self.preview.widget_list = text

    def generate_thread_text(self, thread):
        if not thread.posts:
            return [urwid.Text('No preview available.')]

        pile_contents = []
        pile_contents.append(urwid.Text( [('title', "%s (%d)" % (thread.title, thread.replies)), "\n"] ))
        
        for post in thread.posts:
            #header = ['%d ' % post.id, ('name', post.name), ' ', post.stamp]
            #pile_contents.append(urwid.Text(header))
            pile_contents.append(PostItem(post))
            pile_contents.append(urwid.Padding(urwid.Text(post.text + '\n'), left=3))

        return pile_contents

    def view_thread(self):
        self.set_status("Loading thread...")
        thread = api_kareha.get_thread(self.threadwalker.get_focus()[0].id)
        
        contents = self.generate_thread_text(thread)
        self.postwalker = urwid.SimpleListWalker(contents)
        #urwid.connect_signal(self.postwalker, 'modified', self.update_status)
        
        pile = urwid.ListBox(self.postwalker)
        self.mainframe.contents['body'] = (pile , None)
        self.mode = 'thread'
        
        self.set_status("Ready.")
    
    def view_front(self):
        self.mainframe.contents['body'] = (self.screen_front, None)
        self.mode = 'front'
    
    def show_replydialog(self):
        #widgets = [urwid.Text('New post'), urwid.Edit('Now')]
        
        #dialog = Dialog(urwid.Pile(widgets), self.mainloop, title='Reply')
        dialog = Dialog(urwid.Filler(urwid.Edit()), self.mainloop, title='Reply')
        dialog.show()

class ThreadItem(urwid.WidgetWrap):
    def __init__ (self, thread):
        self.id = thread.id
        self.content = '%d: %s (%s)' % (thread.num, thread.title, thread.replies)
        self.thread = thread

        #self.item = [
        #    ('fixed', 15, urwid.Padding(urwid.AttrWrap(
        #        urwid.Text('item %s' % str(id)), 'body', 'focus'), left=2)),
        #    urwid.AttrWrap(urwid.Text('%s' % description), 'body', 'focus'),
        #]
        #w = urwid.Columns(self.item)

        w = urwid.AttrMap(urwid.Text(self.content), 'body', 'focus')
        self.__super.__init__(w)

    def selectable (self):
        return True

    def keypress(self, size, key):
        return key

class PostItem(urwid.WidgetWrap):
    def __init__ (self, post):
        self.id = post.id
        self.content = ['%d ' % post.id, ('name', post.name), ' ', post.stamp]
        self.post = post

        #w = urwid.AttrMap(urwid.Text(self.content), 'body', 'focus')
        w = urwid.Text(self.content)
        self.__super.__init__(w)

    def selectable (self):
        return False

    def keypress(self, size, key):
        return key

class Dialog(urwid.Overlay):
    def __init__(self, widget, loop, width=30, height=None, title=''):
        self.widget = urwid.AttrMap(urwid.LineBox(widget, title=title), 'window')
        self.oldwidget = loop.widget
        self.loop = loop
        self.__super.__init__(self.widget, loop.widget,
                align="center",
                width=width,
                valign="middle",
                height=height)
    
    def show(self):
        self.loop.widget = self
        
    def close(self):
        self.loop.widget = self.oldwidget
        
    def keypress(self, size, key):
        #if key in ('up', 'down', 'left', 'right', 'enter'):
        #    self.widget.keypress(size, key)
        #elif key == 'esc':
        #    self.close()
        return self.widget.keypress(size, key)

main = KarehaReader()
